!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ######################
      MODULE MODD_DIAG_IDEAL_n
!     ######################
!
!!****  *MODD_DIAG_IDEAL - declaration of diagnostics for IDEAL scheme
!!
!!    PURPOSE
!!    -------
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	P. Le Moigne   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       04/2009
!
!*       0.   DECLARATIONS
!             ------------
!
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE DIAG_IDEAL_t
!------------------------------------------------------------------------------
!
  REAL    :: XDIAG_TSTEP  ! time step for diagnostics writing
!
  INTEGER :: N2M          ! flag for 2 meters (and 10 meters) quantities
  LOGICAL :: L2M_MIN_ZS   ! flag for 2 meters quantities evaluated on
!                         ! the minimum orographyy of the grid      
  LOGICAL :: LSURF_BUDGET ! flag for surface energy budget
  LOGICAL :: LRAD_BUDGET  ! flag for radiative energy budget
  LOGICAL :: LCOEF        ! flag for transfer coefficients
  LOGICAL :: LSURF_VARS   ! flag for surface variables
  LOGICAL :: LSURF_BUDGETC       ! flag for surface cumulated energy budget
  LOGICAL :: LRESET_BUDGETC      ! flag for surface cumulated energy budget

!* averaged variables
!
  REAL, POINTER, DIMENSION(:)   :: XRI      ! Bulk-Richardson number           (-)
  REAL, POINTER, DIMENSION(:)   :: XCD      ! drag coefficient for wind        (W/s2)
  REAL, POINTER, DIMENSION(:)   :: XCH      ! drag coefficient for heat        (W/s)
  REAL, POINTER, DIMENSION(:)   :: XCE      ! drag coefficient for vapor       (W/s/K)
  REAL, POINTER, DIMENSION(:)   :: XZ0      ! roughness length for momentum    (m)
  REAL, POINTER, DIMENSION(:)   :: XZ0H     ! roughness length for heat        (m)
  REAL, POINTER, DIMENSION(:)   :: XRN      ! net radiation at surface         (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XH       ! sensible heat flux               (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XLE      ! latent heat flux                 (W/m2) 
  REAL, POINTER, DIMENSION(:)   :: XGFLUX   ! net soil-vegetation flux         (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XT2M     ! air temperature at 2 meters      (K)
  REAL, POINTER, DIMENSION(:)   :: XQ2M     ! air humidity at 2 meters         (kg/kg)
  REAL, POINTER, DIMENSION(:)   :: XHU2M    ! relative humidity at 2 meters    (-)
  REAL, POINTER, DIMENSION(:)   :: XQS      ! humidity at surface              (kg/kg)
  REAL, POINTER, DIMENSION(:)   :: XZON10M  ! zonal wind at 10 meters          (m/s)
  REAL, POINTER, DIMENSION(:)   :: XMER10M  ! meridian wind at 10 meters       (m/s)
  REAL, POINTER, DIMENSION(:)   :: XLWD     ! downward long wave radiation     (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XLWU     ! upward long wave radiation       (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XSWD     ! downward short wave radiation    (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XSWU     ! upward short wave radiation      (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XSWBD    ! downward short wave radiation by spectral band   (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XSWBU    ! upward short wave radiation by spectral band (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XFMU     ! horizontal momentum flux zonal   (m2/s2)
  REAL, POINTER, DIMENSION(:)   :: XFMV     ! horizontal momentum flux meridian (m2/s2)             
!------------------------------------------------------------------------------
!

END TYPE DIAG_IDEAL_t

TYPE(DIAG_IDEAL_t), ALLOCATABLE, TARGET, SAVE :: DIAG_IDEAL_MODEL(:)

REAL, POINTER :: XDIAG_TSTEP=>NULL()
!$OMP THREADPRIVATE(XDIAG_TSTEP)
INTEGER, POINTER :: N2M=>NULL()
!$OMP THREADPRIVATE(N2M)
LOGICAL, POINTER :: L2M_MIN_ZS=>NULL()
!$OMP THREADPRIVATE(L2M_MIN_ZS)
LOGICAL, POINTER :: LSURF_BUDGET=>NULL()
!$OMP THREADPRIVATE(LSURF_BUDGET)
LOGICAL, POINTER :: LRAD_BUDGET=>NULL()
!$OMP THREADPRIVATE(LRAD_BUDGET)
LOGICAL, POINTER :: LCOEF=>NULL()
!$OMP THREADPRIVATE(LCOEF)
LOGICAL, POINTER :: LSURF_VARS=>NULL()
!$OMP THREADPRIVATE(LSURF_VARS)
LOGICAL, POINTER :: LSURF_BUDGETC=>NULL()
!$OMP THREADPRIVATE(LSURF_BUDGETC)
LOGICAL, POINTER :: LRESET_BUDGETC=>NULL()
!$OMP THREADPRIVATE(LRESET_BUDGETC)
REAL, POINTER, DIMENSION(:)   :: XRI=>NULL()
!$OMP THREADPRIVATE(XRI)
REAL, POINTER, DIMENSION(:)   :: XCD=>NULL()
!$OMP THREADPRIVATE(XCD)
REAL, POINTER, DIMENSION(:)   :: XCH=>NULL()
!$OMP THREADPRIVATE(XCH)
REAL, POINTER, DIMENSION(:)   :: XCE=>NULL()
!$OMP THREADPRIVATE(XCE)
REAL, POINTER, DIMENSION(:)   :: XZ0=>NULL()
!$OMP THREADPRIVATE(XZ0)
REAL, POINTER, DIMENSION(:)   :: XZ0H=>NULL()
!$OMP THREADPRIVATE(XZ0H)
REAL, POINTER, DIMENSION(:)   :: XRN=>NULL()
!$OMP THREADPRIVATE(XRN)
REAL, POINTER, DIMENSION(:)   :: XH=>NULL()
!$OMP THREADPRIVATE(XH)
REAL, POINTER, DIMENSION(:)   :: XLE=>NULL()
!$OMP THREADPRIVATE(XLE)
REAL, POINTER, DIMENSION(:)   :: XGFLUX=>NULL()
!$OMP THREADPRIVATE(XGFLUX)
REAL, POINTER, DIMENSION(:)   :: XT2M=>NULL()
!$OMP THREADPRIVATE(XT2M)
REAL, POINTER, DIMENSION(:)   :: XQ2M=>NULL()
!$OMP THREADPRIVATE(XQ2M)
REAL, POINTER, DIMENSION(:)   :: XHU2M=>NULL()
!$OMP THREADPRIVATE(XHU2M)
REAL, POINTER, DIMENSION(:)   :: XQS=>NULL()
!$OMP THREADPRIVATE(XQS)
REAL, POINTER, DIMENSION(:)   :: XZON10M=>NULL()
!$OMP THREADPRIVATE(XZON10M)
REAL, POINTER, DIMENSION(:)   :: XMER10M=>NULL()
!$OMP THREADPRIVATE(XMER10M)
REAL, POINTER, DIMENSION(:)   :: XLWD=>NULL()
!$OMP THREADPRIVATE(XLWD)
REAL, POINTER, DIMENSION(:)   :: XLWU=>NULL()
!$OMP THREADPRIVATE(XLWU)
REAL, POINTER, DIMENSION(:)   :: XSWD=>NULL()
!$OMP THREADPRIVATE(XSWD)
REAL, POINTER, DIMENSION(:)   :: XSWU=>NULL()
!$OMP THREADPRIVATE(XSWU)
REAL, POINTER, DIMENSION(:,:) :: XSWBD=>NULL()
!$OMP THREADPRIVATE(XSWBD)
REAL, POINTER, DIMENSION(:,:) :: XSWBU=>NULL()
!$OMP THREADPRIVATE(XSWBU)
REAL, POINTER, DIMENSION(:)   :: XFMU=>NULL()
!$OMP THREADPRIVATE(XFMU)
REAL, POINTER, DIMENSION(:)   :: XFMV=>NULL()
!$OMP THREADPRIVATE(XFMV)

CONTAINS

SUBROUTINE DIAG_IDEAL_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
DIAG_IDEAL_MODEL(KFROM)%XRI=>XRI
DIAG_IDEAL_MODEL(KFROM)%XCD=>XCD
DIAG_IDEAL_MODEL(KFROM)%XCH=>XCH
DIAG_IDEAL_MODEL(KFROM)%XCE=>XCE
DIAG_IDEAL_MODEL(KFROM)%XZ0=>XZ0
DIAG_IDEAL_MODEL(KFROM)%XZ0H=>XZ0H
DIAG_IDEAL_MODEL(KFROM)%XRN=>XRN
DIAG_IDEAL_MODEL(KFROM)%XH=>XH
DIAG_IDEAL_MODEL(KFROM)%XLE=>XLE
DIAG_IDEAL_MODEL(KFROM)%XGFLUX=>XGFLUX
DIAG_IDEAL_MODEL(KFROM)%XT2M=>XT2M
DIAG_IDEAL_MODEL(KFROM)%XQ2M=>XQ2M
DIAG_IDEAL_MODEL(KFROM)%XHU2M=>XHU2M
DIAG_IDEAL_MODEL(KFROM)%XQS=>XQS
DIAG_IDEAL_MODEL(KFROM)%XZON10M=>XZON10M
DIAG_IDEAL_MODEL(KFROM)%XMER10M=>XMER10M
DIAG_IDEAL_MODEL(KFROM)%XLWD=>XLWD
DIAG_IDEAL_MODEL(KFROM)%XLWU=>XLWU
DIAG_IDEAL_MODEL(KFROM)%XSWD=>XSWD
DIAG_IDEAL_MODEL(KFROM)%XSWU=>XSWU
DIAG_IDEAL_MODEL(KFROM)%XSWBD=>XSWBD
DIAG_IDEAL_MODEL(KFROM)%XSWBU=>XSWBU
DIAG_IDEAL_MODEL(KFROM)%XFMU=>XFMU
DIAG_IDEAL_MODEL(KFROM)%XFMV=>XFMV
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_IDEAL_N:DIAG_IDEAL_GOTO_MODEL',0,ZHOOK_HANDLE)
XDIAG_TSTEP=>DIAG_IDEAL_MODEL(KTO)%XDIAG_TSTEP
N2M=>DIAG_IDEAL_MODEL(KTO)%N2M
L2M_MIN_ZS=>DIAG_IDEAL_MODEL(KTO)%L2M_MIN_ZS
LSURF_BUDGET=>DIAG_IDEAL_MODEL(KTO)%LSURF_BUDGET
LRAD_BUDGET=>DIAG_IDEAL_MODEL(KTO)%LRAD_BUDGET
LCOEF=>DIAG_IDEAL_MODEL(KTO)%LCOEF
LSURF_VARS=>DIAG_IDEAL_MODEL(KTO)%LSURF_VARS
LSURF_BUDGETC=>DIAG_IDEAL_MODEL(KTO)%LSURF_BUDGETC
LRESET_BUDGETC=>DIAG_IDEAL_MODEL(KTO)%LRESET_BUDGETC
XRI=>DIAG_IDEAL_MODEL(KTO)%XRI
XCD=>DIAG_IDEAL_MODEL(KTO)%XCD
XCH=>DIAG_IDEAL_MODEL(KTO)%XCH
XCE=>DIAG_IDEAL_MODEL(KTO)%XCE
XZ0=>DIAG_IDEAL_MODEL(KTO)%XZ0
XZ0H=>DIAG_IDEAL_MODEL(KTO)%XZ0H
XRN=>DIAG_IDEAL_MODEL(KTO)%XRN
XH=>DIAG_IDEAL_MODEL(KTO)%XH
XLE=>DIAG_IDEAL_MODEL(KTO)%XLE
XGFLUX=>DIAG_IDEAL_MODEL(KTO)%XGFLUX
XT2M=>DIAG_IDEAL_MODEL(KTO)%XT2M
XQ2M=>DIAG_IDEAL_MODEL(KTO)%XQ2M
XHU2M=>DIAG_IDEAL_MODEL(KTO)%XHU2M
XQS=>DIAG_IDEAL_MODEL(KTO)%XQS
XZON10M=>DIAG_IDEAL_MODEL(KTO)%XZON10M
XMER10M=>DIAG_IDEAL_MODEL(KTO)%XMER10M
XLWD=>DIAG_IDEAL_MODEL(KTO)%XLWD
XLWU=>DIAG_IDEAL_MODEL(KTO)%XLWU
XSWD=>DIAG_IDEAL_MODEL(KTO)%XSWD
XSWU=>DIAG_IDEAL_MODEL(KTO)%XSWU
XSWBD=>DIAG_IDEAL_MODEL(KTO)%XSWBD
XSWBU=>DIAG_IDEAL_MODEL(KTO)%XSWBU
XFMU=>DIAG_IDEAL_MODEL(KTO)%XFMU
XFMV=>DIAG_IDEAL_MODEL(KTO)%XFMV
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_IDEAL_N:DIAG_IDEAL_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE DIAG_IDEAL_GOTO_MODEL

SUBROUTINE DIAG_IDEAL_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_IDEAL_N:DIAG_IDEAL_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(DIAG_IDEAL_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(DIAG_IDEAL_MODEL(J)%XRI)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XCD)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XCH)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XCE)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XZ0)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XZ0H)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XRN)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XH)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XLE)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XGFLUX)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XT2M)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XQ2M)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XHU2M)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XQS)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XZON10M)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XMER10M)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XLWD)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XLWU)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XSWD)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XSWU)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XSWBD)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XSWBU)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XFMU)
  NULLIFY(DIAG_IDEAL_MODEL(J)%XFMV)
ENDDO
DIAG_IDEAL_MODEL(:)%XDIAG_TSTEP=0.
DIAG_IDEAL_MODEL(:)%N2M=0
DIAG_IDEAL_MODEL(:)%L2M_MIN_ZS=.FALSE.
DIAG_IDEAL_MODEL(:)%LSURF_BUDGET=.FALSE.
DIAG_IDEAL_MODEL(:)%LRAD_BUDGET=.FALSE.
DIAG_IDEAL_MODEL(:)%LCOEF=.FALSE.
DIAG_IDEAL_MODEL(:)%LSURF_VARS=.FALSE.
DIAG_IDEAL_MODEL(:)%LSURF_BUDGETC=.FALSE.
DIAG_IDEAL_MODEL(:)%LRESET_BUDGETC=.FALSE.
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_IDEAL_N:DIAG_IDEAL_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_IDEAL_ALLOC

SUBROUTINE DIAG_IDEAL_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_IDEAL_N:DIAG_IDEAL_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(DIAG_IDEAL_MODEL)) DEALLOCATE(DIAG_IDEAL_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_IDEAL_N:DIAG_IDEAL_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_IDEAL_DEALLO

END MODULE MODD_DIAG_IDEAL_n
