!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!######################
MODULE MODD_DIAG_ISBA_n
!######################
!
!!****  *MODD_DIAG_ISBA - declaration of diagnostics for ISBA scheme
!!
!!    PURPOSE
!!    -------
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	V. Masson   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       01/2004
!!      Modified    01/2006 : sea flux parameterization.
!
!*       0.   DECLARATIONS
!             ------------
!
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE DIAG_ISBA_t
!------------------------------------------------------------------------------
!
  REAL    :: XDIAG_TSTEP  ! time step for diagnostics writing
!
  INTEGER :: N2M          ! flag for 2 meters (and 10 meters) quantities
  LOGICAL :: L2M_MIN_ZS   ! flag for 2 meters quantities evaluated on
!                         ! the minimum orographyy of the grid      
  LOGICAL :: LSURF_BUDGET   ! flag for surface energy budget
  LOGICAL :: LRAD_BUDGET    ! flag for radiative energy budget
!
  LOGICAL :: LCOEF        ! flag for transfer coefficients
  LOGICAL :: LSURF_VARS   ! flag for surface variables
!
  LOGICAL :: LPGD          ! flag for writing of PGD files
  LOGICAL :: LPATCH_BUDGET ! flag for patch output
!
!* variables for each patch
!
  REAL, POINTER, DIMENSION(:,:) :: XRI     ! Bulk-Richardson number           (-)
  REAL, POINTER, DIMENSION(:,:) :: XCD     ! drag coefficient for wind        (W/s2)
  REAL, POINTER, DIMENSION(:,:) :: XCH     ! drag coefficient for heat        (W/s)
  REAL, POINTER, DIMENSION(:,:) :: XCE     ! drag coefficient for vapor       (W/s/K)
  REAL, POINTER, DIMENSION(:,:) :: XRN     ! net radiation at surface         (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XH      ! sensible heat flux               (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLE     ! total latent heat flux           (W/m2) 
  REAL, POINTER, DIMENSION(:,:) :: XLEI    ! sublimation latent heat flux     (W/m2) 
  REAL, POINTER, DIMENSION(:,:) :: XGFLUX  ! net soil-vegetation flux         (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XTS     ! surface temperature              (K)
  REAL, POINTER, DIMENSION(:,:) :: XTSRAD  ! surface temperature              (K)  
  REAL, POINTER, DIMENSION(:,:) :: XT2M    ! temperature at 2 meters          (K)
  REAL, POINTER, DIMENSION(:,:) :: XQ2M    ! humidity    at 2 meters          (kg/kg)
  REAL, POINTER, DIMENSION(:,:) :: XHU2M   ! relative humidity at 2 meters    (-)
  REAL, POINTER, DIMENSION(:,:) :: XQS     ! humidity at surface              (kg/kg)
  REAL, POINTER, DIMENSION(:,:) :: XZON10M ! zonal wind at 10 meters          (m/s)
  REAL, POINTER, DIMENSION(:,:) :: XMER10M ! meridian wind at 10 meters       (m/s)
  REAL, POINTER, DIMENSION(:,:) :: XWIND10M! wind at 10 meters                (m/s)
  REAL, POINTER, DIMENSION(:,:) :: XLWD    ! downward long wave radiation     (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLWU    ! upward long wave radiation       (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XSWD    ! downward short wave radiation    (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XSWU    ! upward short wave radiation      (W/m2)
  REAL, POINTER, DIMENSION(:,:,:) :: XSWBD ! downward short wave radiation by spectral band   (W/m2)
  REAL, POINTER, DIMENSION(:,:,:) :: XSWBU ! upward short wave radiation by spectral band (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XFMU    ! horizontal momentum flux zonal    (Pa)
  REAL, POINTER, DIMENSION(:,:) :: XFMV    ! horizontal momentum flux meridian (Pa)             
  ! 
  REAL, POINTER, DIMENSION(:,:) :: XSWDC   ! downward short wave radiation     (J/m2)
  REAL, POINTER, DIMENSION(:,:) :: XSWUC   ! upward short wave radiation       (J/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLWDC   ! downward long wave radiation      (J/m2)
  REAL, POINTER, DIMENSION(:,:) :: XLWUC   ! upward long wave radiation        (J/m2)
  REAL, POINTER, DIMENSION(:,:) :: XFMUC   ! horizontal momentum flux zonal    (Pa.s)
  REAL, POINTER, DIMENSION(:,:) :: XFMVC   ! horizontal momentum flux meridian (Pa.s)
  !
  REAL, POINTER, DIMENSION(:,:) :: XZ0_WITH_SNOW  ! roughness length for momentum
                                                  ! for vegetation and snow    (m)
  REAL, POINTER, DIMENSION(:,:) :: XZ0H_WITH_SNOW ! roughness length for heat
                                                  ! for vegetation and snow    (m)
  REAL, POINTER, DIMENSION(:,:) :: XZ0EFF         ! effective roughness length for heat
                                                  ! for vegetation and snow    (m)
!
!* averaged variables
!
  REAL, POINTER, DIMENSION(:)   :: XAVG_RI       ! Bulk-Richardson number           (-)
  REAL, POINTER, DIMENSION(:)   :: XAVG_CD       ! drag coefficient for wind        (W/s2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_CH       ! drag coefficient for heat        (W/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_CE       ! drag coefficient for vapor       (W/s/K)
  REAL, POINTER, DIMENSION(:)   :: XAVG_RN       ! net radiation at surface         (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_H        ! sensible heat flux               (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LE       ! total latent heat flux           (W/m2) 
  REAL, POINTER, DIMENSION(:)   :: XAVG_LEI      ! sublimation latent heat flux     (W/m2) 
  REAL, POINTER, DIMENSION(:)   :: XAVG_GFLUX    ! net soil-vegetation flux         (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_TS       ! surface temperature              (K)
  REAL, POINTER, DIMENSION(:)   :: XAVG_TSRAD    ! surface temperature              (K)  
  REAL, POINTER, DIMENSION(:)   :: XAVG_T2M      ! temperature at 2 meters          (K)
  REAL, POINTER, DIMENSION(:)   :: XAVG_T2M_MIN  ! Minimum temperature at 2 meters          (K)
  REAL, POINTER, DIMENSION(:)   :: XAVG_T2M_MAX  ! Maximum temperature at 2 meters          (K)
  REAL, POINTER, DIMENSION(:)   :: XAVG_Q2M      ! humidity    at 2 meters          (kg/kg)
  REAL, POINTER, DIMENSION(:)   :: XAVG_HU2M     ! relative humidity at 2 meters    (-)
  REAL, POINTER, DIMENSION(:)   :: XAVG_HU2M_MIN ! Minimum relative humidity at 2 meters    (-)
  REAL, POINTER, DIMENSION(:)   :: XAVG_HU2M_MAX ! Maximum relative humidity at 2 meters    (-)
  REAL, POINTER, DIMENSION(:)   :: XAVG_QS       ! humidity at surface              (kg/kg)
  REAL, POINTER, DIMENSION(:)   :: XAVG_ZON10M   ! zonal wind at 10 meters          (m/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_MER10M   ! meridian wind at 10 meters       (m/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_WIND10M  ! wind at 10 meters                (m/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_WIND10M_MAX  ! Maximum wind at 10 meters    (m/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_SFCO2    ! CO2 flux                         (kg/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LWD      ! downward long wave radiation     (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LWU      ! upward long wave radiation       (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_SWD      ! downward short wave radiation    (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_SWU      ! upward short wave radiation      (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XAVG_SWBD     ! downward short wave radiation by spectral band   (W/m2)
  REAL, POINTER, DIMENSION(:,:) :: XAVG_SWBU     ! upward short wave radiation by spectral band (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_FMU      ! horizontal momentum flux zonal   (m2/s2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_FMV      ! horizontal momentum flux meridian (m2/s2) 
  REAL, POINTER, DIMENSION(:)   :: XAVG_LWDC     ! downward long wave radiation     (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_LWUC     ! upward long wave radiation       (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_SWDC     ! downward short wave radiation    (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_SWUC     ! upward short wave radiation      (J/m2)
  REAL, POINTER, DIMENSION(:)   :: XAVG_FMUC     ! horizontal momentum flux zonal   (Pa.s)
  REAL, POINTER, DIMENSION(:)   :: XAVG_FMVC     ! horizontal momentum flux meridian (Pa.s)             
  
  !                                                
  REAL, POINTER, DIMENSION(:)   :: XAVG_Z0       ! roughness length for momentum
                                                 ! for vegetation and snow    (m)
  REAL, POINTER, DIMENSION(:)   :: XAVG_Z0H      ! roughness length for heat
                                                 ! for vegetation and snow    (m)
  REAL, POINTER, DIMENSION(:)   :: XAVG_Z0EFF    ! effective roughness length for heat
                                                 ! for vegetation and snow    (m)
!------------------------------------------------------------------------------
!

END TYPE DIAG_ISBA_t

TYPE(DIAG_ISBA_t), ALLOCATABLE, TARGET, SAVE :: DIAG_ISBA_MODEL(:)

REAL, POINTER :: XDIAG_TSTEP=>NULL()
!$OMP THREADPRIVATE(XDIAG_TSTEP)
INTEGER, POINTER :: N2M=>NULL()
!$OMP THREADPRIVATE(N2M)
LOGICAL, POINTER :: L2M_MIN_ZS=>NULL()
!$OMP THREADPRIVATE(L2M_MIN_ZS)
LOGICAL, POINTER :: LSURF_BUDGET=>NULL()
!$OMP THREADPRIVATE(LSURF_BUDGET)
LOGICAL, POINTER :: LRAD_BUDGET=>NULL()
!$OMP THREADPRIVATE(LRAD_BUDGET)
LOGICAL, POINTER :: LCOEF=>NULL()
!$OMP THREADPRIVATE(LCOEF)
LOGICAL, POINTER :: LSURF_VARS=>NULL()
!$OMP THREADPRIVATE(LSURF_VARS)
LOGICAL, POINTER :: LPGD=>NULL()
!$OMP THREADPRIVATE(LPGD)
LOGICAL, POINTER :: LPATCH_BUDGET=>NULL()
!$OMP THREADPRIVATE(LPATCH_BUDGET)
REAL, POINTER, DIMENSION(:,:) :: XRI=>NULL()
!$OMP THREADPRIVATE(XRI)
REAL, POINTER, DIMENSION(:,:) :: XCD=>NULL()
!$OMP THREADPRIVATE(XCD)
REAL, POINTER, DIMENSION(:,:) :: XCH=>NULL()
!$OMP THREADPRIVATE(XCH)
REAL, POINTER, DIMENSION(:,:) :: XCE=>NULL()
!$OMP THREADPRIVATE(XCE)
REAL, POINTER, DIMENSION(:,:) :: XRN=>NULL()
!$OMP THREADPRIVATE(XRN)
REAL, POINTER, DIMENSION(:,:) :: XH=>NULL()
!$OMP THREADPRIVATE(XH)
REAL, POINTER, DIMENSION(:,:) :: XLE=>NULL()
!$OMP THREADPRIVATE(XLE)
REAL, POINTER, DIMENSION(:,:) :: XLEI=>NULL()
!$OMP THREADPRIVATE(XLEI)
REAL, POINTER, DIMENSION(:,:) :: XGFLUX=>NULL()
!$OMP THREADPRIVATE(XGFLUX)
REAL, POINTER, DIMENSION(:,:) :: XTS=>NULL()
!$OMP THREADPRIVATE(XTS)
REAL, POINTER, DIMENSION(:,:) :: XTSRAD=>NULL()
!$OMP THREADPRIVATE(XTSRAD)
REAL, POINTER, DIMENSION(:,:) :: XT2M=>NULL()
!$OMP THREADPRIVATE(XT2M)
REAL, POINTER, DIMENSION(:,:) :: XQ2M=>NULL()
!$OMP THREADPRIVATE(XQ2M)
REAL, POINTER, DIMENSION(:,:) :: XHU2M=>NULL()
!$OMP THREADPRIVATE(XHU2M)
REAL, POINTER, DIMENSION(:,:) :: XQS=>NULL()
!$OMP THREADPRIVATE(XQS)
REAL, POINTER, DIMENSION(:,:) :: XZON10M=>NULL()
!$OMP THREADPRIVATE(XZON10M)
REAL, POINTER, DIMENSION(:,:) :: XMER10M=>NULL()
!$OMP THREADPRIVATE(XMER10M)
REAL, POINTER, DIMENSION(:,:) :: XWIND10M=>NULL()
!$OMP THREADPRIVATE(XWIND10M)
REAL, POINTER, DIMENSION(:,:) :: XLWD=>NULL()
!$OMP THREADPRIVATE(XLWD)
REAL, POINTER, DIMENSION(:,:) :: XLWU=>NULL()
!$OMP THREADPRIVATE(XLWU)
REAL, POINTER, DIMENSION(:,:) :: XSWD=>NULL()
!$OMP THREADPRIVATE(XSWD)
REAL, POINTER, DIMENSION(:,:) :: XSWU=>NULL()
!$OMP THREADPRIVATE(XSWU)
REAL, POINTER, DIMENSION(:,:,:) :: XSWBD=>NULL()
!$OMP THREADPRIVATE(XSWBD)
REAL, POINTER, DIMENSION(:,:,:) :: XSWBU=>NULL()
!$OMP THREADPRIVATE(XSWBU)
REAL, POINTER, DIMENSION(:,:) :: XFMU=>NULL()
!$OMP THREADPRIVATE(XFMU)
REAL, POINTER, DIMENSION(:,:) :: XFMV=>NULL()
!$OMP THREADPRIVATE(XFMV)
REAL, POINTER, DIMENSION(:,:) :: XLWDC=>NULL()
!$OMP THREADPRIVATE(XLWDC)
REAL, POINTER, DIMENSION(:,:) :: XLWUC=>NULL()
!$OMP THREADPRIVATE(XLWUC)
REAL, POINTER, DIMENSION(:,:) :: XSWDC=>NULL()
!$OMP THREADPRIVATE(XSWDC)
REAL, POINTER, DIMENSION(:,:) :: XSWUC=>NULL()
!$OMP THREADPRIVATE(XSWUC)
REAL, POINTER, DIMENSION(:,:) :: XFMUC=>NULL()
!$OMP THREADPRIVATE(XFMUC)
REAL, POINTER, DIMENSION(:,:) :: XFMVC=>NULL()
!$OMP THREADPRIVATE(XFMVC)
REAL, POINTER, DIMENSION(:,:) :: XZ0_WITH_SNOW=>NULL()
!$OMP THREADPRIVATE(XZ0_WITH_SNOW)
REAL, POINTER, DIMENSION(:,:) :: XZ0H_WITH_SNOW=>NULL()
!$OMP THREADPRIVATE(XZ0H_WITH_SNOW)
REAL, POINTER, DIMENSION(:,:) :: XZ0EFF=>NULL()
!$OMP THREADPRIVATE(XZ0EFF)
REAL, POINTER, DIMENSION(:)   :: XAVG_RI=>NULL()
!$OMP THREADPRIVATE(XAVG_RI)
REAL, POINTER, DIMENSION(:)   :: XAVG_CD=>NULL()
!$OMP THREADPRIVATE(XAVG_CD)
REAL, POINTER, DIMENSION(:)   :: XAVG_CH=>NULL()
!$OMP THREADPRIVATE(XAVG_CH)
REAL, POINTER, DIMENSION(:)   :: XAVG_CE=>NULL()
!$OMP THREADPRIVATE(XAVG_CE)
REAL, POINTER, DIMENSION(:)   :: XAVG_RN=>NULL()
!$OMP THREADPRIVATE(XAVG_RN)
REAL, POINTER, DIMENSION(:)   :: XAVG_H=>NULL()
!$OMP THREADPRIVATE(XAVG_H)
REAL, POINTER, DIMENSION(:)   :: XAVG_LE=>NULL()
!$OMP THREADPRIVATE(XAVG_LE)
REAL, POINTER, DIMENSION(:)   :: XAVG_LEI=>NULL()
!$OMP THREADPRIVATE(XAVG_LEI)
REAL, POINTER, DIMENSION(:)   :: XAVG_GFLUX=>NULL()
!$OMP THREADPRIVATE(XAVG_GFLUX)
REAL, POINTER, DIMENSION(:)   :: XAVG_TS=>NULL()
!$OMP THREADPRIVATE(XAVG_TS)
REAL, POINTER, DIMENSION(:)   :: XAVG_TSRAD=>NULL()
!$OMP THREADPRIVATE(XAVG_TSRAD)
REAL, POINTER, DIMENSION(:)   :: XAVG_T2M=>NULL()
!$OMP THREADPRIVATE(XAVG_T2M)
REAL, POINTER, DIMENSION(:)   :: XAVG_T2M_MIN=>NULL()
!$OMP THREADPRIVATE(XAVG_T2M_MIN)
REAL, POINTER, DIMENSION(:)   :: XAVG_T2M_MAX=>NULL()
!$OMP THREADPRIVATE(XAVG_T2M_MAX)
REAL, POINTER, DIMENSION(:)   :: XAVG_Q2M=>NULL()
!$OMP THREADPRIVATE(XAVG_Q2M)
REAL, POINTER, DIMENSION(:)   :: XAVG_HU2M=>NULL()
!$OMP THREADPRIVATE(XAVG_HU2M)
REAL, POINTER, DIMENSION(:)   :: XAVG_HU2M_MIN=>NULL()
!$OMP THREADPRIVATE(XAVG_HU2M_MIN)
REAL, POINTER, DIMENSION(:)   :: XAVG_HU2M_MAX=>NULL()
!$OMP THREADPRIVATE(XAVG_HU2M_MAX)
REAL, POINTER, DIMENSION(:)   :: XAVG_QS=>NULL()
!$OMP THREADPRIVATE(XAVG_QS)
REAL, POINTER, DIMENSION(:)   :: XAVG_ZON10M=>NULL()
!$OMP THREADPRIVATE(XAVG_ZON10M)
REAL, POINTER, DIMENSION(:)   :: XAVG_MER10M=>NULL()
!$OMP THREADPRIVATE(XAVG_MER10M)
REAL, POINTER, DIMENSION(:)   :: XAVG_WIND10M=>NULL()
!$OMP THREADPRIVATE(XAVG_WIND10M)
REAL, POINTER, DIMENSION(:)   :: XAVG_WIND10M_MAX=>NULL()
!$OMP THREADPRIVATE(XAVG_WIND10M_MAX)
REAL, POINTER, DIMENSION(:)   :: XAVG_SFCO2=>NULL()
!$OMP THREADPRIVATE(XAVG_SFCO2)
REAL, POINTER, DIMENSION(:)   :: XAVG_LWD=>NULL()
!$OMP THREADPRIVATE(XAVG_LWD)
REAL, POINTER, DIMENSION(:)   :: XAVG_LWU=>NULL()
!$OMP THREADPRIVATE(XAVG_LWU)
REAL, POINTER, DIMENSION(:)   :: XAVG_SWD=>NULL()
!$OMP THREADPRIVATE(XAVG_SWD)
REAL, POINTER, DIMENSION(:)   :: XAVG_SWU=>NULL()
!$OMP THREADPRIVATE(XAVG_SWU)
REAL, POINTER, DIMENSION(:,:) :: XAVG_SWBD=>NULL()
!$OMP THREADPRIVATE(XAVG_SWBD)
REAL, POINTER, DIMENSION(:,:) :: XAVG_SWBU=>NULL()
!$OMP THREADPRIVATE(XAVG_SWBU)
REAL, POINTER, DIMENSION(:)   :: XAVG_FMU=>NULL()
!$OMP THREADPRIVATE(XAVG_FMU)
REAL, POINTER, DIMENSION(:)   :: XAVG_FMV=>NULL()
!$OMP THREADPRIVATE(XAVG_FMV)
REAL, POINTER, DIMENSION(:)   :: XAVG_Z0=>NULL()
!$OMP THREADPRIVATE(XAVG_Z0)
REAL, POINTER, DIMENSION(:)   :: XAVG_Z0H=>NULL()
!$OMP THREADPRIVATE(XAVG_Z0H)
REAL, POINTER, DIMENSION(:)   :: XAVG_Z0EFF=>NULL()
!$OMP THREADPRIVATE(XAVG_Z0EFF)
REAL, POINTER, DIMENSION(:)   :: XAVG_LWDC=>NULL()
!$OMP THREADPRIVATE(XAVG_LWDC)
REAL, POINTER, DIMENSION(:)   :: XAVG_LWUC=>NULL()
!$OMP THREADPRIVATE(XAVG_LWUC)
REAL, POINTER, DIMENSION(:)   :: XAVG_SWDC=>NULL()
!$OMP THREADPRIVATE(XAVG_SWDC)
REAL, POINTER, DIMENSION(:)   :: XAVG_SWUC=>NULL()
!$OMP THREADPRIVATE(XAVG_SWUC)
REAL, POINTER, DIMENSION(:)   :: XAVG_FMUC=>NULL()
!$OMP THREADPRIVATE(XAVG_FMUC)
REAL, POINTER, DIMENSION(:)   :: XAVG_FMVC=>NULL()
!$OMP THREADPRIVATE(XAVG_FMVC)

CONTAINS

SUBROUTINE DIAG_ISBA_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
DIAG_ISBA_MODEL(KFROM)%XRI=>XRI
DIAG_ISBA_MODEL(KFROM)%XCD=>XCD
DIAG_ISBA_MODEL(KFROM)%XCH=>XCH
DIAG_ISBA_MODEL(KFROM)%XCE=>XCE
DIAG_ISBA_MODEL(KFROM)%XRN=>XRN
DIAG_ISBA_MODEL(KFROM)%XH=>XH
DIAG_ISBA_MODEL(KFROM)%XLE=>XLE
DIAG_ISBA_MODEL(KFROM)%XLEI=>XLEI
DIAG_ISBA_MODEL(KFROM)%XGFLUX=>XGFLUX
DIAG_ISBA_MODEL(KFROM)%XTS=>XTS
DIAG_ISBA_MODEL(KFROM)%XTSRAD=>XTSRAD
DIAG_ISBA_MODEL(KFROM)%XT2M=>XT2M
DIAG_ISBA_MODEL(KFROM)%XQ2M=>XQ2M
DIAG_ISBA_MODEL(KFROM)%XHU2M=>XHU2M
DIAG_ISBA_MODEL(KFROM)%XQS=>XQS
DIAG_ISBA_MODEL(KFROM)%XZON10M=>XZON10M
DIAG_ISBA_MODEL(KFROM)%XMER10M=>XMER10M
DIAG_ISBA_MODEL(KFROM)%XWIND10M=>XWIND10M
DIAG_ISBA_MODEL(KFROM)%XLWD=>XLWD
DIAG_ISBA_MODEL(KFROM)%XLWU=>XLWU
DIAG_ISBA_MODEL(KFROM)%XSWD=>XSWD
DIAG_ISBA_MODEL(KFROM)%XSWU=>XSWU
DIAG_ISBA_MODEL(KFROM)%XSWBD=>XSWBD
DIAG_ISBA_MODEL(KFROM)%XSWBU=>XSWBU
DIAG_ISBA_MODEL(KFROM)%XFMU=>XFMU
DIAG_ISBA_MODEL(KFROM)%XFMV=>XFMV
DIAG_ISBA_MODEL(KFROM)%XLWDC=>XLWDC
DIAG_ISBA_MODEL(KFROM)%XLWUC=>XLWUC
DIAG_ISBA_MODEL(KFROM)%XSWDC=>XSWDC
DIAG_ISBA_MODEL(KFROM)%XSWUC=>XSWUC
DIAG_ISBA_MODEL(KFROM)%XFMUC=>XFMUC
DIAG_ISBA_MODEL(KFROM)%XFMVC=>XFMVC
DIAG_ISBA_MODEL(KFROM)%XZ0_WITH_SNOW=>XZ0_WITH_SNOW
DIAG_ISBA_MODEL(KFROM)%XZ0H_WITH_SNOW=>XZ0H_WITH_SNOW
DIAG_ISBA_MODEL(KFROM)%XZ0EFF=>XZ0EFF
DIAG_ISBA_MODEL(KFROM)%XAVG_RI=>XAVG_RI
DIAG_ISBA_MODEL(KFROM)%XAVG_CD=>XAVG_CD
DIAG_ISBA_MODEL(KFROM)%XAVG_CH=>XAVG_CH
DIAG_ISBA_MODEL(KFROM)%XAVG_CE=>XAVG_CE
DIAG_ISBA_MODEL(KFROM)%XAVG_RN=>XAVG_RN
DIAG_ISBA_MODEL(KFROM)%XAVG_H=>XAVG_H
DIAG_ISBA_MODEL(KFROM)%XAVG_LE=>XAVG_LE
DIAG_ISBA_MODEL(KFROM)%XAVG_LEI=>XAVG_LEI
DIAG_ISBA_MODEL(KFROM)%XAVG_GFLUX=>XAVG_GFLUX
DIAG_ISBA_MODEL(KFROM)%XAVG_TS=>XAVG_TS
DIAG_ISBA_MODEL(KFROM)%XAVG_TSRAD=>XAVG_TSRAD
DIAG_ISBA_MODEL(KFROM)%XAVG_T2M=>XAVG_T2M
DIAG_ISBA_MODEL(KFROM)%XAVG_T2M_MIN=>XAVG_T2M_MIN
DIAG_ISBA_MODEL(KFROM)%XAVG_T2M_MAX=>XAVG_T2M_MAX
DIAG_ISBA_MODEL(KFROM)%XAVG_Q2M=>XAVG_Q2M
DIAG_ISBA_MODEL(KFROM)%XAVG_HU2M=>XAVG_HU2M
DIAG_ISBA_MODEL(KFROM)%XAVG_HU2M_MIN=>XAVG_HU2M_MIN
DIAG_ISBA_MODEL(KFROM)%XAVG_HU2M_MAX=>XAVG_HU2M_MAX
DIAG_ISBA_MODEL(KFROM)%XAVG_QS=>XAVG_QS
DIAG_ISBA_MODEL(KFROM)%XAVG_ZON10M=>XAVG_ZON10M
DIAG_ISBA_MODEL(KFROM)%XAVG_MER10M=>XAVG_MER10M
DIAG_ISBA_MODEL(KFROM)%XAVG_WIND10M=>XAVG_WIND10M
DIAG_ISBA_MODEL(KFROM)%XAVG_WIND10M_MAX=>XAVG_WIND10M_MAX
DIAG_ISBA_MODEL(KFROM)%XAVG_SFCO2=>XAVG_SFCO2
DIAG_ISBA_MODEL(KFROM)%XAVG_LWD=>XAVG_LWD
DIAG_ISBA_MODEL(KFROM)%XAVG_LWU=>XAVG_LWU
DIAG_ISBA_MODEL(KFROM)%XAVG_SWD=>XAVG_SWD
DIAG_ISBA_MODEL(KFROM)%XAVG_SWU=>XAVG_SWU
DIAG_ISBA_MODEL(KFROM)%XAVG_SWBD=>XAVG_SWBD
DIAG_ISBA_MODEL(KFROM)%XAVG_SWBU=>XAVG_SWBU
DIAG_ISBA_MODEL(KFROM)%XAVG_FMU=>XAVG_FMU
DIAG_ISBA_MODEL(KFROM)%XAVG_FMV=>XAVG_FMV
DIAG_ISBA_MODEL(KFROM)%XAVG_Z0=>XAVG_Z0
DIAG_ISBA_MODEL(KFROM)%XAVG_Z0H=>XAVG_Z0H
DIAG_ISBA_MODEL(KFROM)%XAVG_Z0EFF=>XAVG_Z0EFF
DIAG_ISBA_MODEL(KFROM)%XAVG_LWDC=>XAVG_LWDC
DIAG_ISBA_MODEL(KFROM)%XAVG_LWUC=>XAVG_LWUC
DIAG_ISBA_MODEL(KFROM)%XAVG_SWDC=>XAVG_SWDC
DIAG_ISBA_MODEL(KFROM)%XAVG_SWUC=>XAVG_SWUC
DIAG_ISBA_MODEL(KFROM)%XAVG_FMUC=>XAVG_FMUC
DIAG_ISBA_MODEL(KFROM)%XAVG_FMVC=>XAVG_FMVC
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_ISBA_N:DIAG_ISBA_GOTO_MODEL',0,ZHOOK_HANDLE)
XDIAG_TSTEP=>DIAG_ISBA_MODEL(KTO)%XDIAG_TSTEP
N2M=>DIAG_ISBA_MODEL(KTO)%N2M
L2M_MIN_ZS=>DIAG_ISBA_MODEL(KTO)%L2M_MIN_ZS
LSURF_BUDGET=>DIAG_ISBA_MODEL(KTO)%LSURF_BUDGET
LRAD_BUDGET=>DIAG_ISBA_MODEL(KTO)%LRAD_BUDGET
LCOEF=>DIAG_ISBA_MODEL(KTO)%LCOEF
LSURF_VARS=>DIAG_ISBA_MODEL(KTO)%LSURF_VARS
LPGD=>DIAG_ISBA_MODEL(KTO)%LPGD
LPATCH_BUDGET=>DIAG_ISBA_MODEL(KTO)%LPATCH_BUDGET
XRI=>DIAG_ISBA_MODEL(KTO)%XRI
XCD=>DIAG_ISBA_MODEL(KTO)%XCD
XCH=>DIAG_ISBA_MODEL(KTO)%XCH
XCE=>DIAG_ISBA_MODEL(KTO)%XCE
XRN=>DIAG_ISBA_MODEL(KTO)%XRN
XH=>DIAG_ISBA_MODEL(KTO)%XH
XLE=>DIAG_ISBA_MODEL(KTO)%XLE
XLEI=>DIAG_ISBA_MODEL(KTO)%XLEI
XGFLUX=>DIAG_ISBA_MODEL(KTO)%XGFLUX
XTS=>DIAG_ISBA_MODEL(KTO)%XTS
XTSRAD=>DIAG_ISBA_MODEL(KTO)%XTSRAD
XT2M=>DIAG_ISBA_MODEL(KTO)%XT2M
XQ2M=>DIAG_ISBA_MODEL(KTO)%XQ2M
XHU2M=>DIAG_ISBA_MODEL(KTO)%XHU2M
XQS=>DIAG_ISBA_MODEL(KTO)%XQS
XZON10M=>DIAG_ISBA_MODEL(KTO)%XZON10M
XMER10M=>DIAG_ISBA_MODEL(KTO)%XMER10M
XWIND10M=>DIAG_ISBA_MODEL(KTO)%XWIND10M
XLWD=>DIAG_ISBA_MODEL(KTO)%XLWD
XLWU=>DIAG_ISBA_MODEL(KTO)%XLWU
XSWD=>DIAG_ISBA_MODEL(KTO)%XSWD
XSWU=>DIAG_ISBA_MODEL(KTO)%XSWU
XSWBD=>DIAG_ISBA_MODEL(KTO)%XSWBD
XSWBU=>DIAG_ISBA_MODEL(KTO)%XSWBU
XFMU=>DIAG_ISBA_MODEL(KTO)%XFMU
XFMV=>DIAG_ISBA_MODEL(KTO)%XFMV
XLWDC=>DIAG_ISBA_MODEL(KTO)%XLWDC
XLWUC=>DIAG_ISBA_MODEL(KTO)%XLWUC
XSWDC=>DIAG_ISBA_MODEL(KTO)%XSWDC
XSWUC=>DIAG_ISBA_MODEL(KTO)%XSWUC
XFMUC=>DIAG_ISBA_MODEL(KTO)%XFMUC
XFMVC=>DIAG_ISBA_MODEL(KTO)%XFMVC
XZ0_WITH_SNOW=>DIAG_ISBA_MODEL(KTO)%XZ0_WITH_SNOW
XZ0H_WITH_SNOW=>DIAG_ISBA_MODEL(KTO)%XZ0H_WITH_SNOW
XZ0EFF=>DIAG_ISBA_MODEL(KTO)%XZ0EFF
XAVG_RI=>DIAG_ISBA_MODEL(KTO)%XAVG_RI
XAVG_CD=>DIAG_ISBA_MODEL(KTO)%XAVG_CD
XAVG_CH=>DIAG_ISBA_MODEL(KTO)%XAVG_CH
XAVG_CE=>DIAG_ISBA_MODEL(KTO)%XAVG_CE
XAVG_RN=>DIAG_ISBA_MODEL(KTO)%XAVG_RN
XAVG_H=>DIAG_ISBA_MODEL(KTO)%XAVG_H
XAVG_LE=>DIAG_ISBA_MODEL(KTO)%XAVG_LE
XAVG_LEI=>DIAG_ISBA_MODEL(KTO)%XAVG_LEI
XAVG_GFLUX=>DIAG_ISBA_MODEL(KTO)%XAVG_GFLUX
XAVG_TS=>DIAG_ISBA_MODEL(KTO)%XAVG_TS
XAVG_TSRAD=>DIAG_ISBA_MODEL(KTO)%XAVG_TSRAD
XAVG_T2M=>DIAG_ISBA_MODEL(KTO)%XAVG_T2M
XAVG_T2M_MIN=>DIAG_ISBA_MODEL(KTO)%XAVG_T2M_MIN
XAVG_T2M_MAX=>DIAG_ISBA_MODEL(KTO)%XAVG_T2M_MAX
XAVG_Q2M=>DIAG_ISBA_MODEL(KTO)%XAVG_Q2M
XAVG_HU2M=>DIAG_ISBA_MODEL(KTO)%XAVG_HU2M
XAVG_HU2M_MIN=>DIAG_ISBA_MODEL(KTO)%XAVG_HU2M_MIN
XAVG_HU2M_MAX=>DIAG_ISBA_MODEL(KTO)%XAVG_HU2M_MAX
XAVG_QS=>DIAG_ISBA_MODEL(KTO)%XAVG_QS
XAVG_ZON10M=>DIAG_ISBA_MODEL(KTO)%XAVG_ZON10M
XAVG_MER10M=>DIAG_ISBA_MODEL(KTO)%XAVG_MER10M
XAVG_WIND10M=>DIAG_ISBA_MODEL(KTO)%XAVG_WIND10M
XAVG_WIND10M_MAX=>DIAG_ISBA_MODEL(KTO)%XAVG_WIND10M_MAX
XAVG_SFCO2=>DIAG_ISBA_MODEL(KTO)%XAVG_SFCO2
XAVG_LWD=>DIAG_ISBA_MODEL(KTO)%XAVG_LWD
XAVG_LWU=>DIAG_ISBA_MODEL(KTO)%XAVG_LWU
XAVG_SWD=>DIAG_ISBA_MODEL(KTO)%XAVG_SWD
XAVG_SWU=>DIAG_ISBA_MODEL(KTO)%XAVG_SWU
XAVG_SWBD=>DIAG_ISBA_MODEL(KTO)%XAVG_SWBD
XAVG_SWBU=>DIAG_ISBA_MODEL(KTO)%XAVG_SWBU
XAVG_FMU=>DIAG_ISBA_MODEL(KTO)%XAVG_FMU
XAVG_FMV=>DIAG_ISBA_MODEL(KTO)%XAVG_FMV
XAVG_Z0=>DIAG_ISBA_MODEL(KTO)%XAVG_Z0
XAVG_Z0H=>DIAG_ISBA_MODEL(KTO)%XAVG_Z0H
XAVG_Z0EFF=>DIAG_ISBA_MODEL(KTO)%XAVG_Z0EFF
XAVG_LWDC=>DIAG_ISBA_MODEL(KTO)%XAVG_LWDC
XAVG_LWUC=>DIAG_ISBA_MODEL(KTO)%XAVG_LWUC
XAVG_SWDC=>DIAG_ISBA_MODEL(KTO)%XAVG_SWDC
XAVG_SWUC=>DIAG_ISBA_MODEL(KTO)%XAVG_SWUC
XAVG_FMUC=>DIAG_ISBA_MODEL(KTO)%XAVG_FMUC
XAVG_FMVC=>DIAG_ISBA_MODEL(KTO)%XAVG_FMVC
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_ISBA_N:DIAG_ISBA_GOTO_MODEL',1,ZHOOK_HANDLE)


END SUBROUTINE DIAG_ISBA_GOTO_MODEL

SUBROUTINE DIAG_ISBA_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_ISBA_N:DIAG_ISBA_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(DIAG_ISBA_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(DIAG_ISBA_MODEL(J)%XRI)
  NULLIFY(DIAG_ISBA_MODEL(J)%XCD)
  NULLIFY(DIAG_ISBA_MODEL(J)%XCH)
  NULLIFY(DIAG_ISBA_MODEL(J)%XCE)
  NULLIFY(DIAG_ISBA_MODEL(J)%XRN)
  NULLIFY(DIAG_ISBA_MODEL(J)%XH)
  NULLIFY(DIAG_ISBA_MODEL(J)%XLE)
  NULLIFY(DIAG_ISBA_MODEL(J)%XLEI)
  NULLIFY(DIAG_ISBA_MODEL(J)%XGFLUX)
  NULLIFY(DIAG_ISBA_MODEL(J)%XTS)
  NULLIFY(DIAG_ISBA_MODEL(J)%XTSRAD)
  NULLIFY(DIAG_ISBA_MODEL(J)%XT2M)
  NULLIFY(DIAG_ISBA_MODEL(J)%XQ2M)
  NULLIFY(DIAG_ISBA_MODEL(J)%XHU2M)
  NULLIFY(DIAG_ISBA_MODEL(J)%XQS)
  NULLIFY(DIAG_ISBA_MODEL(J)%XZON10M)
  NULLIFY(DIAG_ISBA_MODEL(J)%XMER10M)
  NULLIFY(DIAG_ISBA_MODEL(J)%XWIND10M)
  NULLIFY(DIAG_ISBA_MODEL(J)%XLWD)
  NULLIFY(DIAG_ISBA_MODEL(J)%XLWU)
  NULLIFY(DIAG_ISBA_MODEL(J)%XSWD)
  NULLIFY(DIAG_ISBA_MODEL(J)%XSWU)
  NULLIFY(DIAG_ISBA_MODEL(J)%XSWBD)
  NULLIFY(DIAG_ISBA_MODEL(J)%XSWBU)
  NULLIFY(DIAG_ISBA_MODEL(J)%XFMU)
  NULLIFY(DIAG_ISBA_MODEL(J)%XFMV)
  NULLIFY(DIAG_ISBA_MODEL(J)%XSWDC)
  NULLIFY(DIAG_ISBA_MODEL(J)%XSWUC)
  NULLIFY(DIAG_ISBA_MODEL(J)%XLWDC)
  NULLIFY(DIAG_ISBA_MODEL(J)%XLWUC)
  NULLIFY(DIAG_ISBA_MODEL(J)%XFMUC)
  NULLIFY(DIAG_ISBA_MODEL(J)%XFMVC)
  NULLIFY(DIAG_ISBA_MODEL(J)%XZ0_WITH_SNOW)
  NULLIFY(DIAG_ISBA_MODEL(J)%XZ0H_WITH_SNOW)
  NULLIFY(DIAG_ISBA_MODEL(J)%XZ0EFF)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_RI)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_CD)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_CH)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_CE)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_RN)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_H)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_LE)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_LEI)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_GFLUX)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_TS)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_TSRAD)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_T2M)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_T2M_MIN)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_T2M_MAX)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_Q2M)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_HU2M)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_HU2M_MIN)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_HU2M_MAX)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_QS)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_ZON10M)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_MER10M)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_WIND10M)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_WIND10M_MAX)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_SFCO2)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_LWD)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_LWU)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_SWD)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_SWU)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_SWBD)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_SWBU)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_FMU)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_FMV)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_LWDC)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_LWUC)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_SWDC)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_SWUC)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_FMUC)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_FMVC)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_Z0)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_Z0H)
  NULLIFY(DIAG_ISBA_MODEL(J)%XAVG_Z0EFF)
ENDDO
DIAG_ISBA_MODEL(:)%XDIAG_TSTEP=0.
DIAG_ISBA_MODEL(:)%N2M=0
DIAG_ISBA_MODEL(:)%L2M_MIN_ZS=.FALSE.
DIAG_ISBA_MODEL(:)%LSURF_BUDGET=.FALSE.
DIAG_ISBA_MODEL(:)%LRAD_BUDGET=.FALSE.
DIAG_ISBA_MODEL(:)%LCOEF=.FALSE.
DIAG_ISBA_MODEL(:)%LSURF_VARS=.FALSE.
DIAG_ISBA_MODEL(:)%LPGD=.FALSE.
DIAG_ISBA_MODEL(:)%LPATCH_BUDGET=.FALSE.
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_ISBA_N:DIAG_ISBA_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_ISBA_ALLOC

SUBROUTINE DIAG_ISBA_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_ISBA_N:DIAG_ISBA_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(DIAG_ISBA_MODEL)) DEALLOCATE(DIAG_ISBA_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_ISBA_N:DIAG_ISBA_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_ISBA_DEALLO

END MODULE MODD_DIAG_ISBA_n
