!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     #########
      SUBROUTINE WRITESURF_PGD_ISBA_PAR_n(HPROGRAM)
!     ################################################
!
!!****  *WRITESURF_PGD_ISBA_PAR_n* - writes ISBA physiographic fields
!!                        
!!
!!    PURPOSE
!!    -------
!!
!!**  METHOD
!!    ------
!!
!!    EXTERNAL
!!    --------
!!
!!
!!    IMPLICIT ARGUMENTS
!!    ------------------
!!
!!    REFERENCE
!!    ---------
!!
!!
!!    AUTHOR
!!    ------
!!	V. Masson   *Meteo France*	
!!
!!    MODIFICATIONS
!!    -------------
!!      Original    01/2003 
!!      P. Le Moigne 12/2004 : add type of photosynthesis 
!-------------------------------------------------------------------------------
!
!*       0.    DECLARATIONS
!              ------------
!
USE MODD_DATA_ISBA_n,    ONLY : NTIME, XPAR_VEG, XPAR_LAI,XPAR_RSMIN,XPAR_GAMMA,XPAR_WRMAX_CF, &
                                XPAR_RGL,XPAR_CV,XPAR_DG,XPAR_Z0,XPAR_Z0_O_Z0H,         &
                                XPAR_ALBNIR_VEG,XPAR_ALBVIS_VEG, XPAR_ALBUV_VEG,          &
                                XPAR_ALBNIR_SOIL,XPAR_ALBVIS_SOIL, XPAR_ALBUV_SOIL,       &
                                XPAR_EMIS, XPAR_DICE,                                     &
                                XPAR_VEGTYPE,XPAR_ROOTFRAC,                                &
                                XPAR_GMES,XPAR_BSLAI,XPAR_LAIMIN,XPAR_SEFOLD,XPAR_GC,   &
                                XPAR_DMAX, XPAR_F2I, LDATA_STRESS, XPAR_H_TREE,XPAR_RE25,&
                                XPAR_CE_NITRO,XPAR_CF_NITRO,XPAR_CNA_NITRO, &
                                XPAR_GROUND_DEPTH, XPAR_ROOT_DEPTH,               &
                                XPAR_ROOT_EXTINCTION, XPAR_ROOT_LIN,              &
                                LPAR_STRESS, XPAR_IRRIG, XPAR_WATSUP, &
                                LDATA_VEGTYPE, LDATA_LAI, LDATA_H_TREE, LDATA_DG, LDATA_ROOTFRAC,&  
                                LDATA_VEG, LDATA_Z0, LDATA_EMIS, LDATA_DICE, &
                                LDATA_RSMIN, LDATA_GAMMA, LDATA_WRMAX_CF, LDATA_RGL, &
                                LDATA_CV, LDATA_Z0_O_Z0H, &
                                LDATA_ALBNIR_VEG, LDATA_ALBVIS_VEG, LDATA_ALBUV_VEG, &
                                LDATA_ALBVIS_SOIL, LDATA_ALBNIR_SOIL, LDATA_ALBUV_SOIL, &
                                LDATA_GMES, LDATA_BSLAI, LDATA_SEFOLD, LDATA_GC, LDATA_DMAX, &
                                LDATA_RE25, LDATA_LAIMIN, LDATA_F2I, &
                                LDATA_CE_NITRO,LDATA_CF_NITRO, LDATA_CNA_NITRO,&
                                LDATA_STRESS, LDATA_IRRIG, LDATA_WATSUP,             &   
                                LDATA_GROUND_DEPTH, LDATA_ROOT_DEPTH,             &
                                LDATA_ROOT_EXTINCTION, LDATA_ROOT_LIN
  
!
USE MODI_WRITE_SURF
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE
!
!*       0.1   Declarations of arguments
!              -------------------------
!
 CHARACTER(LEN=6),  INTENT(IN)  :: HPROGRAM ! program calling
!
!*       0.2   Declarations of local variables
!              -------------------------------
!
INTEGER           :: IRESP          ! IRESP  : return-code if a problem appears
 CHARACTER(LEN=12) :: YRECFM         ! Name of the article to be read
 CHARACTER(LEN=100):: YCOMMENT       ! Comment string
INTEGER           :: JTIME          ! loop index
INTEGER           :: JLAYER         ! loop index
INTEGER           :: JPATCH         ! loop index
REAL,    DIMENSION(:,:), ALLOCATABLE :: ZWORK
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
!-------------------------------------------------------------------------------
!
IF (LHOOK) CALL DR_HOOK('WRITESURF_PGD_ISBA_PAR_N',0,ZHOOK_HANDLE)
YRECFM='L_VEGTYPE'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_VEGTYPE,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_VEGTYPE) THEN
  YRECFM='D_VEGTYPE'
  YCOMMENT='X_Y_DATA_VEGTYPE'
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_VEGTYPE(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
IF (LDATA_LAI .OR. LDATA_VEG .OR. LDATA_Z0 .OR. LDATA_EMIS) THEN
  YRECFM='NDATA_TIME'
  YCOMMENT='(-)'
  CALL WRITE_SURF(HPROGRAM,YRECFM,NTIME,IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_VEG'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_VEG,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_VEG) THEN
  DO JTIME=1,NTIME
    WRITE(YRECFM,FMT='(A7,I2.2)') 'D_VEG_T',JTIME
    YCOMMENT='X_Y_D_VEG'
    CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_VEG(:,JTIME,:),IRESP,HCOMMENT=YCOMMENT)
  END DO
ENDIF
!
YRECFM='L_LAI'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_LAI,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_LAI) THEN
  DO JTIME=1,NTIME
    WRITE(YRECFM,FMT='(A7,I2.2)') 'D_LAI_T',JTIME
    YCOMMENT='X_Y_D_LAI'
    CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_LAI(:,JTIME,:),IRESP,HCOMMENT=YCOMMENT)
  END DO
ENDIF
!
YRECFM='L_Z0'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_Z0,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_Z0) THEN
  DO JTIME=1,NTIME
    WRITE(YRECFM,FMT='(A6,I2.2)') 'D_Z0_T',JTIME
    YCOMMENT='X_Y_D_Z0'
    CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_Z0(:,JTIME,:),IRESP,HCOMMENT=YCOMMENT)
  END DO
ENDIF
!
YRECFM='L_EMIS'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_EMIS,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_EMIS) THEN
  DO JTIME=1,NTIME
    WRITE(YRECFM,FMT='(A8,I2.2)') 'D_EMIS_T',JTIME
    YCOMMENT='X_Y_D_EMIS'
    CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_EMIS(:,JTIME,:),IRESP,HCOMMENT=YCOMMENT)
  END DO
ENDIF
!
YRECFM='L_RSMIN'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_RSMIN,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_RSMIN) THEN
  YRECFM='D_RSMIN'
  YCOMMENT='X_Y_D_RSMIN'
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_RSMIN(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_GAMMA'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_GAMMA,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_GAMMA) THEN
  YRECFM='D_GAMMA'
  YCOMMENT='X_Y_D_GAMMA'
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_GAMMA(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_WRMAX_CF'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_WRMAX_CF,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_WRMAX_CF) THEN
  YRECFM='D_WRMAX_CF'
  YCOMMENT='X_Y_D_WRMAX_CF'
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_WRMAX_CF(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_RGL'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_RGL,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_RGL) THEN
  YRECFM='D_RGL'
  YCOMMENT='X_Y_D_RGL'
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_RGL(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_CV'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_CV,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_CV) THEN
  YRECFM='D_CV'
  YCOMMENT='X_Y_D_CV'
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_CV(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_Z0_O_Z0H'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_Z0_O_Z0H,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_Z0_O_Z0H) THEN
  YRECFM='D_Z0_O_Z0H'
  YCOMMENT='X_Y_D_Z0_O_Z0H'
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_Z0_O_Z0H(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_DG'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_DG,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_DG) THEN
  ALLOCATE(ZWORK(SIZE(XPAR_DG,1),SIZE(XPAR_DG,3)))
  DO JLAYER=1,SIZE(XPAR_DG,2)
    IF (JLAYER<10)  WRITE(YRECFM,FMT='(A4,I1.1)') 'D_DG',JLAYER
    IF (JLAYER>=10) WRITE(YRECFM,FMT='(A4,I2.2)') 'D_DG',JLAYER
    YCOMMENT='X_Y_'//YRECFM
    DO JPATCH=1,SIZE(XPAR_DG,3)
      ZWORK(:,JPATCH) = XPAR_DG(:,JLAYER,JPATCH)
    END DO
    CALL WRITE_SURF(HPROGRAM,YRECFM,ZWORK,IRESP,HCOMMENT=YCOMMENT)
  END DO
  DEALLOCATE(ZWORK)
ENDIF
!
YRECFM='L_ROOTFRAC'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_ROOTFRAC,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_ROOTFRAC) THEN
  ALLOCATE(ZWORK(SIZE(XPAR_ROOTFRAC,1),SIZE(XPAR_ROOTFRAC,3)))
  DO JLAYER=1,SIZE(XPAR_ROOTFRAC,2)
    IF (JLAYER<10)  WRITE(YRECFM,FMT='(A10,I1.1)') 'D_ROOTFRAC',JLAYER
    IF (JLAYER>=10) WRITE(YRECFM,FMT='(A10,I2.2)') 'D_ROOTFRAC',JLAYER
    YCOMMENT='X_Y_'//YRECFM
    DO JPATCH=1,SIZE(XPAR_ROOTFRAC,3)
      ZWORK(:,JPATCH) = XPAR_ROOTFRAC(:,JLAYER,JPATCH)
    END DO
    CALL WRITE_SURF(HPROGRAM,YRECFM,ZWORK,IRESP,HCOMMENT=YCOMMENT)
  END DO
  DEALLOCATE(ZWORK)
ENDIF
!
YRECFM='L_GROUND_DPT'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_GROUND_DEPTH,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_GROUND_DEPTH) THEN
  YRECFM='D_GROUND_DPT'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_GROUND_DEPTH(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_ROOT_DEPTH'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_ROOT_DEPTH,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_ROOT_DEPTH) THEN
  YRECFM='D_ROOT_DEPTH'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_ROOT_DEPTH(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_ROOT_EXT'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_ROOT_EXTINCTION,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_ROOT_EXTINCTION) THEN
  YRECFM='D_ROOT_EXT'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_ROOT_EXTINCTION(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_ROOT_LIN'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_ROOT_LIN,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_ROOT_LIN) THEN
  YRECFM='D_ROOT_LIN'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_ROOT_LIN(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_DICE'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_DICE,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_DICE) THEN
  YRECFM='D_DICE'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_DICE(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_ALBNIR_VEG'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_ALBNIR_VEG,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_ALBNIR_VEG) THEN
  YRECFM='D_ALBNIR_VEG'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_ALBNIR_VEG(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_ALBVIS_VEG'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_ALBVIS_VEG,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_ALBVIS_VEG) THEN
  YRECFM='D_ALBVIS_VEG'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_ALBVIS_VEG(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_ALBUV_VEG'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_ALBUV_VEG,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_ALBUV_VEG) THEN
  YRECFM='D_ALBUV_VEG'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_ALBUV_VEG(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_ALBNIR_SOI'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_ALBNIR_SOIL,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_ALBNIR_SOIL) THEN
  YRECFM='D_ALBNIR_SOI'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_ALBNIR_SOIL(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_ALBVIS_SOI'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_ALBVIS_SOIL,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_ALBVIS_SOIL) THEN
  YRECFM='D_ALBVIS_SOI'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_ALBVIS_SOIL(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_ALBUV_SOI'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_ALBUV_SOIL,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_ALBUV_SOIL) THEN
  YRECFM='D_ALBUV_SOI'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_ALBUV_SOIL(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_GMES'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_GMES,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_GMES) THEN
  YRECFM='D_GMES'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_GMES(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_BSLAI'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_BSLAI,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_BSLAI) THEN
  YRECFM='D_BSLAI'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_BSLAI(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_LAIMIN'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_LAIMIN,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_LAIMIN) THEN
  YRECFM='D_LAIMIN'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_LAIMIN(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_SEFOLD'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_SEFOLD,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_SEFOLD) THEN
  YRECFM='D_SEFOLD'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_SEFOLD(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_GC'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_GC,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_GC) THEN
  YRECFM='D_GC'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_GC(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_DMAX'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_DMAX,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_DMAX) THEN
  YRECFM='D_DMAX'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_DMAX(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_F2I'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_F2I,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_F2I) THEN
  YRECFM='D_F2I'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_F2I(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_STRESS'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_STRESS,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_STRESS) THEN
  ALLOCATE(ZWORK(SIZE(LPAR_STRESS,1),SIZE(LPAR_STRESS,2)))
  ZWORK=0.
  WHERE(LPAR_STRESS) ZWORK=1.
  YRECFM='D_STRESS'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,ZWORK(:,:),IRESP,HCOMMENT=YCOMMENT)
  DEALLOCATE(ZWORK)
ENDIF
!
YRECFM='L_H_TREE'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_H_TREE,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_H_TREE) THEN
  YRECFM='D_H_TREE'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_H_TREE(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_RE25'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_RE25,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_RE25) THEN
  YRECFM='D_RE25'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_RE25(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_CE_NITRO'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_CE_NITRO,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_CE_NITRO) THEN
  YRECFM='D_CE_NITRO'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_CE_NITRO(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_CF_NITRO'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_CF_NITRO,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_CF_NITRO) THEN
  YRECFM='D_CF_NITRO'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_CF_NITRO(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_CNA_NITRO'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_CNA_NITRO,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_CNA_NITRO) THEN
  YRECFM='D_CNA_NITRO'
  YCOMMENT='X_Y_'//YRECFM
  CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_CNA_NITRO(:,:),IRESP,HCOMMENT=YCOMMENT)
ENDIF
!
YRECFM='L_IRRIG'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_IRRIG,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_IRRIG) THEN
  DO JTIME=1,NTIME
    WRITE(YRECFM,FMT='(A9,I2.2)') 'D_IRRIG_T',JTIME
    YCOMMENT='X_Y_IRRIG'
    CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_IRRIG(:,JTIME,:),IRESP,HCOMMENT=YCOMMENT)
  ENDDO
ENDIF
!
YRECFM='L_WATSUP'
YCOMMENT=YRECFM
 CALL WRITE_SURF(HPROGRAM,YRECFM,LDATA_WATSUP,IRESP,HCOMMENT=YCOMMENT)
IF (LDATA_WATSUP) THEN
  DO JTIME=1,NTIME
    WRITE(YRECFM,FMT='(A10,I2.2)') 'D_WATSUP_T',JTIME
    YCOMMENT='X_Y_WATSUP'
    CALL WRITE_SURF(HPROGRAM,YRECFM,XPAR_WATSUP(:,JTIME,:),IRESP,HCOMMENT=YCOMMENT)
  ENDDO
ENDIF
!
IF (LHOOK) CALL DR_HOOK('WRITESURF_PGD_ISBA_PAR_N',1,ZHOOK_HANDLE)
!
!-------------------------------------------------------------------------------
!
END SUBROUTINE WRITESURF_PGD_ISBA_PAR_n
